/**
 * Copyright (c) 2008-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.tools.rmi.common.serializable;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import javax.xml.transform.stream.StreamSource;

import junit.framework.TestCase;

import org.junit.Test;
import org.objectweb.petals.tools.rmi.common.serializable.NormalizedMessage;

import com.ebmwebsourcing.easycommons.xml.SourceHelper;
import com.ebmwebsourcing.easycommons.xml.XMLComparator;

/**
 * @author Adrien Ruffie - EBM WebSourcing
 */
public class NormalizedMessageTest {

    private final static String XML = "<?xml version=\"1.0\" encoding=\"UTF-8\"?><source><title>XSL</title><author>John Smith</author></source>";

    private static final String getTestXmlContent(int padLength) {
        final char padChar = '-';
        StringBuffer sb = new StringBuffer("<?xml version=\"1.0\" encoding=\"UTF-8\"?><source>");
        for (int i = 0; i < padLength; ++i) {
            sb.append(padChar);
        }
        sb.append("</source>");
        return sb.toString();
    }

    private void testWriteReadObjectDelegateOfNormalizedMessageContent(String testXmlContent)
            throws Exception {
        final NormalizedMessage outputedNM = new NormalizedMessage();
        outputedNM
                .setContent(new StreamSource(new ByteArrayInputStream(testXmlContent.getBytes())));
        final ByteArrayOutputStream baos = new ByteArrayOutputStream();
        final ObjectOutputStream oos = new ObjectOutputStream(baos);
        oos.writeObject(outputedNM);
        oos.close();
        final ObjectInputStream ois = new ObjectInputStream(new ByteArrayInputStream(
                baos.toByteArray()));
        final NormalizedMessage inputedNM = (NormalizedMessage) ois.readObject();
        ois.close();

        TestCase.assertTrue("Both normalized message must be equals", XMLComparator.isEquivalent(
                SourceHelper.toString(inputedNM.getContent()), testXmlContent));
    }

    @Test
    public void testWriteReadObjectDelegateOfNormalizedMessageContentSimpleXmlContent()
            throws Exception {
        testWriteReadObjectDelegateOfNormalizedMessageContent(XML);
    }

    @Test
    public void testWriteReadObjectDelegateOfNormalizedMessageContentXmlContentOfVariousSize()
            throws Exception {
        for (int i = 0; i < 1024 * 10; ++i) {
            testWriteReadObjectDelegateOfNormalizedMessageContent(getTestXmlContent(i));
        }
    }
}
